#include <stdlib.h>
#include <stdio.h>
#include <glib.h>
#include <time.h>

/*Simple struct definition*/
struct Point
{
	int X;
	int Y;
};

/*Initialize the Data to be processed by the lists*/
inline struct Point* initialize_data(int size)
{
	struct Point* dataptr = malloc(sizeof(struct Point) * size);
	for(int i = 0; i < size; ++i)
	{
		dataptr[i].X = i;
		dataptr[i].Y = i;
	}
	return dataptr;
}

/*Initialize the indexes where elements will be inserted/removed*/
inline int* initialize_indexes(int size, int operationcount, int counter, int denominator)
{
	int* indexesptr = malloc(operationcount * sizeof(int));
	
	srand(time(NULL));
	for(int i = 0; i < operationcount; ++i)
	{
		int value = 0;
		int containsalready = 0;
		
		do
		{
			containsalready = 0;
			value = (counter - 1) *(size/denominator) +  rand() % (size/denominator);
			
			for(int j = 0; j < i; ++j)
				if(indexesptr[j] == value)
				{
					containsalready = 1;
				}
		}while(containsalready);
		indexesptr[i] = value;
	}
	
	return indexesptr;
}

/*Initialize the Arraylist*/
inline GArray* initialize_arraylist(int size, struct Point* dataptr)
{
	GArray* arraylist = g_array_new(FALSE, FALSE, sizeof(struct Point));
	
	for(int i = 0; i < size; ++i)
	{
		arraylist = g_array_append_val(arraylist, dataptr[i]);
	}
	
	return arraylist;
}


int main(int argc, char* argv[])
{
	/*params*/
	if(argc < 6)
		{
			printf("Wrong arguments!\n[1]listsizes\n[2]remove/add(0/1)\n[3]operationcount\n[4]counter\n[5]denominator\n");
			return -1;
		}
	int size = atoi(argv[1]);
	int remove = atoi(argv[2]);
	int operationcount = atoi(argv[3]);
	int counter = atoi(argv[4]);
	int denominator = atoi(argv[5]);
	
	
	/*initialization*/
	struct Point* allValues = initialize_data(size);
	int* indexes = initialize_indexes(size, operationcount, counter, denominator);
	GArray* arraylist = initialize_arraylist(size, allValues);
	
	/*Benchmark*/
	clock_t time;
	if(remove)
	{
		/*Remove Benchmark*/
		time = clock();
		for(int i = 0; i < operationcount; ++i)
		{
			guint len = arraylist->len;
			for(int j = 0; j < len; ++j)
			{
				struct Point* p = &g_array_index(arraylist,struct Point, j);
				if(p->X == allValues[indexes[i]].X)
				{
					arraylist = g_array_remove_index(arraylist, j);
					break;
				}
			}
		}
		time = clock() - time;
		printf("%f\t", ((float)time)/CLOCKS_PER_SEC);
	}
	else
	{
		/*Insert Benchmark*/
		time = clock();
		for(int i = 0; i < operationcount; ++i)
		{
			guint len = arraylist->len;
			for(int j = 0; j < len; ++j)
			{
				struct Point* p = &g_array_index(arraylist,struct Point, j);
				if(p->X == allValues[indexes[i]].X && p->Y ==allValues[indexes[i]].Y)
				{
					/*
					This isn't optimal,the malloc isn't needed and it should be freed.
					However this way both Array and Linkedlist do the same operations.
					*/
					struct Point* p = malloc(sizeof(struct Point));
					p->X = 1337;
					p->Y = 1337;
					arraylist = g_array_insert_val(arraylist, j, *p);
					break;
				}
			}
		}
		time = clock() - time;
		printf("%f\t", ((float)time)/CLOCKS_PER_SEC);
	}
	

	return 0;
}