#include <stdlib.h>
#include <stdio.h>
#include <glib.h>
#include <time.h>

/*Simple struct definition*/
struct Point
{
	int X;
	int Y;
};

/*Initialize the Data to be processed by the lists*/
inline struct Point* initialize_data(int size)
{
	struct Point* dataptr = malloc(sizeof(struct Point) * size);
	for(int i = 0; i < size; ++i)
	{
		dataptr[i].X = i;
		dataptr[i].Y = i;
	}
	return dataptr;
}

/*Initialize the indexes where elements will be inserted/removed*/
inline int* initialize_indexes(int size, int operationcount, int counter, int denominator)
{
	int* indexesptr = malloc(operationcount * sizeof(int));
	
	srand(time(NULL));
	for(int i = 0; i < operationcount; ++i)
	{
		int value = 0;
		int containsalready = 0;
		
		do
		{
			containsalready = 0;
			value = (counter - 1) *(size/denominator) +  rand() % (size/denominator);
			
			for(int j = 0; j < i; ++j)
				if(indexesptr[j] == value)
				{
					containsalready = 1;
				}
		}while(containsalready);
		indexesptr[i] = value;
	}
	
	return indexesptr;
}

/*Initialize the Linkedlist*/
inline GList* initialize_linkedlist(int size, struct Point* dataptr)
{
	GList* linkedlist = NULL;
	for(int i = size-1; i >= 0; --i)
	{
		linkedlist = g_list_prepend(linkedlist, &dataptr[i]);
	}
	
	return linkedlist;
}


int main(int argc, char* argv[])
{
	/*params*/
	if(argc < 6)
		{
			printf("Wrong arguments!\n[1]listsizes\n[2]remove/add(0/1)\n[3]operationcount\n[4]counter\n[5]denominator\n");
			return -1;
		}
	int size = atoi(argv[1]);
	int remove = atoi(argv[2]);
	int operationcount = atoi(argv[3]);
	int counter = atoi(argv[4]);
	int denominator = atoi(argv[5]);
	
	
	/*initialization*/
	struct Point* allValues = initialize_data(size);
	int* indexes = initialize_indexes(size, operationcount, counter, denominator);
	GList* linkedlist = initialize_linkedlist(size, allValues);
	
	/*Benchmark*/
	clock_t time;
	if(remove)
	{
		/*Remove Benchmark*/
		time = clock();
		for(int i = 0; i < operationcount; ++i)
		{
			linkedlist = g_list_remove(linkedlist, &allValues[indexes[i]]);
		}
		time = clock() - time;
		printf("%f\n", ((float)time)/CLOCKS_PER_SEC);
		
	}
	else
	{
		/*Insert Benchmark*/
		time = clock();
		for(int i = 0; i < operationcount; ++i)
		{
			struct Point* p = malloc(sizeof(struct Point));
			p->X = 1337;
			p->Y = 1337;
			GList* sibling = g_list_find(linkedlist, &allValues[indexes[i]]);
			linkedlist = g_list_insert_before(linkedlist, sibling, p);
		}
		time = clock() - time;
		printf("%f\n", ((float)time)/CLOCKS_PER_SEC);
	}

	return 0;
}
